@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2016-2019 Christian Jaeger, copying@christianjaeger.ch
+# Copyright (c) 2016-2022 Christian Jaeger, copying@christianjaeger.ch
 #
 # This is free software, offered under either the same terms as perl 5
 # or the terms of the Artistic License version 2 or the terms of the
@@ -7,6 +7,8 @@
 # bundled with this file.
 #
 
+# Depends: (sensors)
+
 =head1 NAME
 
 Chj::Linux::LmSensors
@@ -51,7 +53,7 @@ use Chj::TEST;
     package Chj::Linux::LmSensors::Selector;
     use FP::Predicates qw(is_string);
 
-    use FP::Struct [[*is_string, 'groupname'], [*is_string, 'sensorname'],],
+    use FP::Struct [[\&is_string, 'groupname'], [\&is_string, 'sensorname'],],
         'FP::Struct::Show', 'FP::Abstract::Pure';
     _END_
 }
