@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2019-2020 Christian Jaeger, copying@christianjaeger.ch
+# Copyright (c) 2019-2021 Christian Jaeger, copying@christianjaeger.ch
 #
 # This is free software, offered under either the same terms as perl 5
 # or the terms of the Artistic License version 2 or the terms of the
@@ -137,6 +137,8 @@ failure protocol (FP::Abstract::Failure) instead?
 
 =head1 SEE ALSO
 
+L<FP::Either> (which wraps both cases in a shared parent type).
+
 Implements: L<FP::Abstract::Pure>, L<FP::Struct::Show>
 
 =head1 NOTE
@@ -175,7 +177,7 @@ package FP::Failure::Failure {
     use FP::Struct [
         "value",
         [
-            *maybe_array, "maybe_parents"
+            \&maybe_array, "maybe_parents"
 
                 # Array of failures that are the reason for this
                 # failure. Values other than FP::Failure::Failure are
@@ -266,7 +268,7 @@ sub failure {
             }
         : undef
     );
-    defined wantarray ? $v : die $v
+    defined wantarray ? $v : die $v    ## no critic
 }
 
 sub is_failure {
@@ -274,13 +276,12 @@ sub is_failure {
     force($_[0])->$_isa("FP::Failure::Failure")
 }
 
-our $use_failure = 0;    # bool
+our $use_failure = 0;                  # bool
 
 sub fails {
     @_ >= 1 and @_ <= 2 or fp_croak_arity "1-2";
-    $use_failure            ? &failure(@_)
-        : defined wantarray ? 0
-        :                     die "fails called in void context";
+    $use_failure ? &failure(@_) : defined wantarray ? 0    ## no critic
+        :          die "fails called in void context";
 }
 
 package FP::Failure::Abstract::Message {
