@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2015-2020 Christian Jaeger, copying@christianjaeger.ch
+# Copyright (c) 2015-2021 Christian Jaeger, copying@christianjaeger.ch
 #
 # This is free software, offered under either the same terms as perl 5
 # or the terms of the Artistic License version 2 or the terms of the
@@ -24,13 +24,13 @@ FP::Optional - dealing with optional values
     if (my ($r) = perhaps_div 10, 2) {
         is $r, 5;
     }
-    *maybe_div = perhaps_to_maybe *perhaps_div;
+    *maybe_div = perhaps_to_maybe \&perhaps_div;
     is maybe_div(10, 2), 5;
     is maybe_div(10, 0), undef;
 
     use FP::Div qw(square);
     # short-cutting evaluation for undef:
-    *optionally_square = optionally(*square);
+    *optionally_square = optionally(\&square);
     is optionally_square(2), 4;
     is optionally_square(undef), undef;
 
@@ -141,14 +141,14 @@ It also offers functions to build chains that propagate failures:
 
 =over 4
 
-=item optionally (*f [, $pos])
+=item optionally (\&f [, $pos])
 
 Returns a function that when receiving undef as its first argument, or
 $_[$pos] if $pos given, directly returns undef without calling f;
 otherwise calls f with the original arguments (with tail-call
 optimization).
 
-=item poptionally (*f)
+=item poptionally (\&f)
 
 Returns a function that when not receiving any argument, directly
 returns (). Otherwise calls it with the original arguments (tail-call
@@ -188,10 +188,11 @@ use Exporter "import";
 our @EXPORT = qw();
 our @EXPORT_OK
     = qw(perhaps_to_maybe perhaps_to_x perhaps_to_or perhaps_to_exists
-    optionally poptionally);
+    optionally poptionally have);
 our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 
 use FP::Carp;
+use FP::Docstring;
 
 # Functions to change the kind of optionals API:
 
@@ -279,4 +280,12 @@ sub poptionally {
     }
 }
 
+sub have {
+    __ 'Turn a "perhaps" (one or more values in list context meaning
+        there is a result, none meaning a failure) into a boolean. In
+        other words, `have` returns true if receiving at least one
+        argument.';
+    @_ ? 1 : ''
+}
+
 1
