@@ -181,6 +181,7 @@ use FP::Show;
 use FP::fix;
 use Carp;
 use FP::Carp;
+use FP::Docstring;
 
 sub stream_iota {
     @_ <= 2 or fp_croak_arity 2;
@@ -935,14 +936,14 @@ sub stream_sort {
 
 TEST {
     require FP::Ops;
-    stream(5, 3, 8, 4)->sort (\&FP::Ops::number_cmp)->array
+    stream(5, 3, 8, 4)->sort (\&FP::Ops::real_cmp)->array
 }
 [3, 4, 5, 8];
 
-TEST { ref(stream(5, 3, 8, 4)->sort (\&FP::Ops::number_cmp)) }
+TEST { ref(stream(5, 3, 8, 4)->sort (\&FP::Ops::real_cmp)) }
 'FP::_::PureArray';    # XX ok? Need to `->stream` if a stream is needed
 
-TEST { stream(5, 3, 10, 8, 4)->sort (\&FP::Ops::number_cmp)->stream->car }
+TEST { stream(5, 3, 10, 8, 4)->sort (\&FP::Ops::real_cmp)->stream->car }
 3;
 
 # but then PureArray has `first`, too, if that's all you need.
@@ -956,12 +957,17 @@ sub stream_group;
 *stream_group = FP::List::make_group(1);
 
 sub FP::List::List::stream_group {
+    __
+        'group($self, $equal, $maybe_tail): build groups of subsequent items that are $equal.';
     @_ >= 2 and @_ <= 3 or fp_croak_arity "2-3";
     my ($self, $equal, $maybe_tail) = @_;
     weaken $_[0];
     stream_group($equal, $self, $maybe_tail)
 }
 
+sub stream_split;
+*stream_split = FP::List::make_split(1);
+
 sub stream_mixed_flatten {
     @_ >= 1 and @_ <= 3 or fp_croak_arity "1-3";
     my ($v, $maybe_tail, $maybe_delay) = @_;
@@ -973,17 +979,18 @@ sub stream_mixed_flatten {
 *FP::List::List::stream_mixed_flatten = \&stream_mixed_flatten;
 
 sub stream_any {
-    @_ == 2 or fp_croak_arity 2;
-    my ($pred, $l) = @_;
+    @_ == 2 or @_ == 3 or fp_croak_arity "2 or 3";
+    my ($pred, $l, $prev_false) = @_;
     weaken $_[1];
     $l = force $l;
     if (is_pair $l) {
-        (&$pred(car $l)) or do {
+        my $v = $pred->(car $l);
+        $v or do {
             my $r = cdr $l;
-            stream_any($pred, $r)
+            stream_any($pred, $r, $v)
         }
     } elsif (is_null $l) {
-        0
+        $prev_false
     } else {
         die "improper list: $l"
     }
@@ -1273,7 +1280,7 @@ TEST {
 TEST {
     stream_any sub { $_[0] % 2 }, array_to_stream []
 }
-0;
+undef;
 TEST {
     stream_any sub { $_[0] % 2 }, array_to_stream [2, 5, 8]
 }
